@@ -1,8 +1,8 @@
 /*
- * $Id: collection.c,v 1.76 2001/12/19 21:21:29 tal197 Exp $
+ * $Id: collection.c,v 1.80 2002/01/09 12:42:33 tal197 Exp $
  *
  * Collection - a GTK+ widget
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * The collection widget provides an area for displaying a collection of
  * objects (such as files). It allows the user to choose a selection of
@@ -39,6 +39,8 @@
 #define MINIMUM_ITEMS 16
 #define AUTOSCROLL_STEP 20
 
+#define MAX_WINKS 3		/* Should be an odd number */
+
 /* Macro to emit the "selection_changed" signal only if allowed */
 #define EMIT_SELECTION_CHANGED(collection, time) \
 	if (!collection->block_selection_changed) \
@@ -552,6 +554,19 @@ static void collection_size_request(GtkWidget *widget,
 #endif
 }
 
+#ifdef GTK2
+static gboolean scroll_after_alloc(Collection *collection)
+{
+	if (collection->wink_item != -1)
+		scroll_to_show(collection, collection->wink_item);
+	else if (collection->cursor_item != -1)
+		scroll_to_show(collection, collection->cursor_item);
+	g_object_unref(G_OBJECT(collection));
+
+	return FALSE;
+}
+#endif
+
 static void collection_size_allocate(GtkWidget *widget,
 				GtkAllocation *allocation)
 {
@@ -617,6 +632,12 @@ static void collection_size_allocate(GtkWidget *widget,
 		/* Need to go around again... */
 		gtk_widget_queue_resize(widget);
 	}
+	else if (collection->wink_item != -1 || collection->cursor_item != -1)
+	{
+		/* Viewport resets the adjustments after the alloc */
+		g_object_ref(G_OBJECT(collection));
+		g_idle_add((GSourceFunc) scroll_after_alloc, collection);
+	}
 #endif
 }
 
@@ -669,8 +690,12 @@ static void clear_area(Collection *collection, GdkRectangle *area)
 				area->width, area->height);
 	}
 	else
+	{
+		gdk_window_set_background(widget->window,
+				&widget->style->bg[GTK_STATE_NORMAL]);
 		gdk_window_clear_area(widget->window,
 				area->x, area->y, area->width, area->height);
+	}
 }
 
 /* Return the area occupied by the item at (row, col) by filling
@@ -1959,11 +1984,26 @@ void collection_qsort(Collection *collection,
 	gpointer cursor_data = NULL;
 	gpointer wink_data = NULL;
 	gpointer wink_on_map_data = NULL;
+	CollectionItem *array;
+	int	i;
 	
 	g_return_if_fail(collection != NULL);
 	g_return_if_fail(IS_COLLECTION(collection));
 	g_return_if_fail(compar != NULL);
 
+	/* Check to see if it needs sorting (saves redrawing) */
+	if (collection->number_of_items < 2)
+		return;
+
+	array = collection->items;
+	for (i = 1; i < collection->number_of_items; i++)
+	{
+		if (compar(&array[i - 1], &array[i]) > 0)
+			break;
+	}
+	if (i == collection->number_of_items)
+		return;		/* Already sorted */
+
 	items = collection->number_of_items;
 
 	wink_on_map = collection->wink_on_map;
@@ -2141,7 +2181,7 @@ void collection_wink_item(Collection *collection, gint item)
 	}
 
 	collection->cursor_item_old = collection->wink_item = item;
-	collection->winks_left = 3;
+	collection->winks_left = MAX_WINKS;
 
 	collection->wink_timeout = gtk_timeout_add(70,
 					   (GtkFunction) wink_timeout,
