@@ -1,8 +1,8 @@
 /*
- * $Id: display.c,v 1.66 2001/12/28 14:47:36 tal197 Exp $
+ * $Id: display.c,v 1.70 2002/01/19 16:47:26 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -567,6 +567,8 @@ void display_set_thumbs(FilerWindow *filer_window, gboolean thumbs)
 		filer_cancel_thumbnails(filer_window);
 
 	filer_set_title(filer_window);
+
+	filer_create_thumbs(filer_window);
 }
 
 /* Set the 'Show Hidden' flag for this window */
@@ -594,9 +596,6 @@ void display_set_autoselect(FilerWindow *filer_window, guchar *leaf)
 	g_return_if_fail(filer_window != NULL);
 	col = filer_window->collection;
 	
-	g_free(filer_window->auto_select);
-	filer_window->auto_select = NULL;
-
 	for (i = 0; i < col->number_of_items; i++)
 	{
 		DirItem *item = (DirItem *) col->items[i].data;
@@ -607,11 +606,16 @@ void display_set_autoselect(FilerWindow *filer_window, guchar *leaf)
 				collection_set_cursor_item(col, i);
 			else
 				collection_wink_item(col, i);
-			return;
+			leaf = NULL;
+			goto out;
 		}
 	}
 	
-	filer_window->auto_select = g_strdup(leaf);
+out:
+	if (leaf)
+		leaf = g_strdup(leaf);
+	g_free(filer_window->auto_select);
+	filer_window->auto_select = leaf;
 }
 
 gboolean display_is_truncated(FilerWindow *filer_window, int i)
@@ -1326,12 +1330,7 @@ void display_update_view(FilerWindow *filer_window,
 	static PangoFontDescription *monospace = NULL;
 
 	if (!monospace)
-	{
-		/* XXX: Should just request 'monospace', but that doesn't
-		 * work for some reason (pango bug?).
-		 */
-		monospace = pango_font_description_from_string("Monotype.com");
-	}
+		monospace = pango_font_description_from_string("monospace");
 	
 	if (view->details)
 	{
@@ -1393,25 +1392,12 @@ void display_update_view(FilerWindow *filer_window,
 	if (filer_window->show_thumbs && item->base_type == TYPE_FILE &&
 		strcmp(item->mime_type->media_type, "image") == 0)
 	{
-		gboolean found;
 		gchar    *path;
 
 		path = make_path(filer_window->path, item->leafname)->str;
 
 		view->image = g_fscache_lookup_full(pixmap_cache, path,
-			FSCACHE_LOOKUP_ONLY_NEW, &found);
-
-		/* If we didn't get an image, it could be because:
-		 *
-		 * - We're loading the image now. found is TRUE,
-		 *   and we'll update the item later.
-		 * - We tried to load the image and failed. found
-		 *   is TRUE.
-		 * - We haven't tried loading the image. found is
-		 *   FALSE, and we start creating the thumb here.
-		 */
-		if (view->image == NULL && !found)
-			filer_create_thumb(filer_window, path);
+				FSCACHE_LOOKUP_ONLY_NEW, NULL);
 	}
 
 	if (!view->image)
