@@ -1,8 +1,8 @@
 /*
- * $Id: dnd.c,v 1.85 2001/12/26 20:33:21 tal197 Exp $
+ * $Id: dnd.c,v 1.88 2002/01/20 16:42:29 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -927,6 +927,14 @@ static void got_data_raw(GtkWidget 		*widget,
 
 	dest_path = g_dataset_get_data(context, "drop_dest_path");
 
+	if (context->action == GDK_ACTION_ASK)
+	{
+		gtk_drag_finish(context, FALSE, FALSE, time);	/* Failure */
+		delayed_error(_("Sorry, can't display a menu of actions "
+				"for a remote file / raw data."));
+		return;
+	}
+
 	if (g_dataset_get_data(context, "drop_dest_type") == drop_dest_prog)
 	{
 		/* The data needs to be sent to an application */
@@ -1004,7 +1012,6 @@ static void got_uri_list(GtkWidget 		*widget,
 		/* There is one URI in the list, and it's not on the local
 		 * machine. Get it via the X server if possible.
 		 */
-		 /* XXX: Action ask? */
 
 		if (provides(context, application_octet_stream))
 		{
@@ -1246,11 +1253,15 @@ static gboolean spring_now(gpointer data)
 	else
 	{
 		spring_window = filer_opendir(dest_path, spring_src_window);
-		gtk_timeout_add(500, spring_check_idle, NULL);
-		gtk_signal_connect(GTK_OBJECT(spring_window->window), "destroy",
-				GTK_SIGNAL_FUNC(spring_win_destroyed), NULL);
 		if (spring_window)
+		{
+			gtk_timeout_add(500, spring_check_idle, NULL);
+			gtk_signal_connect(GTK_OBJECT(spring_window->window),
+					"destroy",
+					GTK_SIGNAL_FUNC(spring_win_destroyed),
+					NULL);
 			centre_window(spring_window->window->window, x, y);
+		}
 	}
 	o_unique_filer_windows = old_unique;
 
