@@ -1,8 +1,8 @@
 /*
- * $Id: filer.c,v 1.235 2001/12/28 14:47:36 tal197 Exp $
+ * $Id: filer.c,v 1.242 2002/01/21 17:03:51 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -403,42 +403,17 @@ static void update_display(Directory *dir,
 {
 	int	old_num;
 	int	i;
-	int	cursor = filer_window->collection->cursor_item;
-	char	*as;
 	Collection *collection = filer_window->collection;
 
 	switch (action)
 	{
 		case DIR_ADD:
-			as = filer_window->auto_select;
-
 			old_num = collection->number_of_items;
 			for (i = 0; i < items->len; i++)
 			{
 				DirItem *item = (DirItem *) items->pdata[i];
 
 				add_item(filer_window, item);
-
-				if (cursor != -1 || !as)
-					continue;
-
-				if (strcmp(as, item->leafname) != 0)
-					continue;
-
-				cursor = collection->number_of_items - 1;
-				if (filer_window->had_cursor)
-				{
-					collection_set_cursor_item(collection,
-							cursor);
-					filer_window->mini_cursor_base = cursor;
-					filer_window->had_cursor = FALSE;
-				}
-				else
-					collection_wink_item(collection,
-							cursor);
-
-				g_free(filer_window->auto_select);
-				filer_window->auto_select = NULL;
 			}
 
 			if (old_num != collection->number_of_items)
@@ -462,15 +437,23 @@ static void update_display(Directory *dir,
 				gdk_window_set_cursor(
 						filer_window->window->window,
 						NULL);
+			set_scanning_display(filer_window, FALSE);
+			toolbar_update_info(filer_window);
+			open_filer_window(filer_window);
+
 			if (filer_window->had_cursor &&
 					collection->cursor_item == -1)
 			{
 				collection_set_cursor_item(collection, 0);
 				filer_window->had_cursor = FALSE;
 			}
-			set_scanning_display(filer_window, FALSE);
-			toolbar_update_info(filer_window);
-			open_filer_window(filer_window);
+			if (filer_window->auto_select)
+				display_set_autoselect(filer_window,
+						filer_window->auto_select);
+			g_free(filer_window->auto_select);
+			filer_window->auto_select = NULL;
+
+			filer_create_thumbs(filer_window);
 
 			if (filer_window->thumb_queue)
 				start_thumb_scanning(filer_window);
@@ -540,6 +523,8 @@ static void filer_window_destroyed(GtkWidget 	*widget,
 		g_list_free(filer_window->thumb_queue);
 	}
 
+	filer_tooltip_prime(NULL, NULL);
+
 	g_free(filer_window->auto_select);
 	g_free(filer_window->path);
 	g_free(filer_window);
@@ -744,6 +729,9 @@ void filer_openitem(FilerWindow *filer_window, int item_number, OpenFlags flags)
 		return;
 	}
 
+	if (!item->image)
+		dir_update_item(filer_window->directory, item->leafname);
+
 	if (item->base_type == TYPE_DIRECTORY)
 	{
 		/* Never close a filer window when opening a directory
@@ -1030,6 +1018,7 @@ static gint key_press_event(GtkWidget	*widget,
 			FilerWindow	*filer_window)
 {
 	gboolean handled;
+	guint key = event->keyval;
 	char group[2] = "1";
 
 	window_with_focus = filer_window;
@@ -1054,7 +1043,7 @@ static gint key_press_event(GtkWidget	*widget,
 	if (handled)
 		return TRUE;
 
-	switch (event->keyval)
+	switch (key)
 	{
 		case GDK_Escape:
 			filer_target_mode(filer_window, NULL, NULL, NULL);
@@ -1077,10 +1066,13 @@ static gint key_press_event(GtkWidget	*widget,
 					filer_window->collection->cursor_item);
 			break;
 		default:
-			if (event->keyval < GDK_0 || event->keyval > GDK_9)
+			if (key >= GDK_0 && key <= GDK_9)
+				group[0] = key - GDK_0 + '0';
+			else if (key >= GDK_KP_0 && key <= GDK_KP_9)
+				group[0] = key - GDK_KP_0 + '0';
+			else
 				return FALSE;
 
-			group[0] = event->keyval - GDK_0 + '0';
 
 			if (event->state & GDK_CONTROL_MASK)
 				group_save(filer_window, group);
@@ -1512,6 +1504,9 @@ static void filer_add_widgets(FilerWindow *filer_window)
 		gtk_box_pack_start(GTK_BOX(vbox), viewport, TRUE, TRUE, 0);
 		filer_window->scrollbar = gtk_vscrollbar_new(adj);
 		gtk_widget_set_usize(viewport, 4, 4);
+
+		gtk_container_set_resize_mode(GTK_CONTAINER(viewport),
+						GTK_RESIZE_IMMEDIATE);
 	}
 #else
 	gtk_box_pack_start(GTK_BOX(vbox), collection, TRUE, TRUE, 0);
@@ -1570,6 +1565,7 @@ static void filer_add_signals(FilerWindow *filer_window)
 	{
 		{"text/uri-list", 0, TARGET_URI_LIST},
 		{"STRING", 0, TARGET_STRING},
+		{"COMPOUND_TEXT", 0, TARGET_STRING},/* XXX: Treats as STRING */
 	};
 
 	/* Events on the top-level window */
@@ -2396,3 +2392,45 @@ void filer_create_thumb(FilerWindow *filer_window, gchar *path)
 
 	start_thumb_scanning(filer_window);
 }
+
+/* If thumbnail display is on, look through all the items in this directory
+ * and start creating or updating the thumbnails as needed.
+ */
+void filer_create_thumbs(FilerWindow *filer_window)
+{
+	Collection *collection = filer_window->collection;
+	int i;
+
+	if (!filer_window->show_thumbs)
+		return;
+
+	for (i = 0; i < collection->number_of_items; i++)
+	{
+		DirItem	 *item = (DirItem *) collection->items[i].data;
+		gchar    *path;
+		gboolean found;
+
+		 if (item->base_type != TYPE_FILE)
+			 continue;
+
+		 if (strcmp(item->mime_type->media_type, "image") != 0)
+			 continue;
+
+		path = make_path(filer_window->path, item->leafname)->str;
+
+		g_fscache_lookup_full(pixmap_cache, path,
+				FSCACHE_LOOKUP_ONLY_NEW, &found);
+
+		/* If we didn't get an image, it could be because:
+		 *
+		 * - We're loading the image now. found is TRUE,
+		 *   and we'll update the item later.
+		 * - We tried to load the image and failed. found
+		 *   is TRUE.
+		 * - We haven't tried loading the image. found is
+		 *   FALSE, and we start creating the thumb here.
+		 */
+		if (!found)
+			filer_create_thumb(filer_window, path);
+	}
+}
