@@ -1,8 +1,8 @@
 /*
- * $Id: gui_support.c,v 1.51 2001/12/26 20:41:05 tal197 Exp $
+ * $Id: gui_support.c,v 1.55 2002/01/16 17:36:45 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -187,9 +187,12 @@ int get_choice(char *title,
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
 
+#ifdef GTK2
+	/* (note: highlights button at the other end!) */
+	gtk_dialog_set_default_response(GTK_DIALOG(dialog), 0);
+#else
 	gtk_window_set_focus(GTK_WINDOW(dialog), button);
 	gtk_window_set_default(GTK_WINDOW(dialog), button);
-#ifndef GTK2
 	gtk_container_set_focus_child(GTK_CONTAINER(action_area), button);
 #endif
 
@@ -245,6 +248,8 @@ void report_error(char *message, ...)
 				GTK_BUTTONS_OK,
 				"%s", s);
 		gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+		gtk_dialog_set_default_response(GTK_DIALOG(dialog),
+						GTK_RESPONSE_OK);
 		gtk_dialog_run(GTK_DIALOG(dialog));
 		gtk_widget_destroy(dialog);
 	}
@@ -369,7 +374,7 @@ void delayed_error(char *error, ...)
 	if (old)
 	{
 		delayed_error_data = g_strconcat(old,
-				_("\n--- next error ---\n"),
+				_("\n---\n"),
 				new, NULL);
 		g_free(old);
 		g_free(new);
@@ -481,6 +486,14 @@ void parse_file(char *path, ParseFunc *parse_line)
 		char *line = data;
 		int  line_number = 1;
 
+		if (strncmp(data, "<?xml ", 6) == 0)
+		{
+			delayed_error(_("Attempt to read an XML file as "
+					"a text file. File '%s' may be "
+					"corrupted."), path);
+			return;
+		}
+
 		while (line && *line)
 		{
 			eol = strchr(line, '\n');
