@@ -1,8 +1,8 @@
 /*
- * $Id: icon.c,v 1.35 2001/12/24 12:13:04 tal197 Exp $
+ * $Id: icon.c,v 1.37 2002/01/16 17:58:45 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -175,33 +175,6 @@ void icon_init(void)
 	tooltips = gtk_tooltips_new();
 }
 
-/* Removes trailing / chars and converts a leading '~/' (if any) to
- * the user's home dir. g_free() the result.
- */
-guchar *icon_convert_path(guchar *path)
-{
-	guchar		*retval;
-	int		path_len;
-
-	g_return_val_if_fail(path != NULL, NULL);
-
-	path_len = strlen(path);
-	while (path_len > 1 && path[path_len - 1] == '/')
-		path_len--;
-	
-	retval = g_strndup(path, path_len);
-
-	if (path[0] == '~' && (path[1] == '\0' || path[1] == '/'))
-	{
-		guchar *tmp = retval;
-
-		retval = g_strconcat(home_dir, retval + 1, NULL);
-		g_free(tmp);
-	}
-
-	return retval;
-}
-
 /* The icons_hash table allows us to convert from a path to a list
  * of icons that use that path.
  * Add this icon to the list for its path.
@@ -601,9 +574,6 @@ static void rename_activate(GtkWidget *dialog)
 	else if (*new_src == '\0')
 		report_error(
 			_("The location must contain at least one character!"));
-	else if (strpbrk(new_name, "<>"))
-		report_error(
-			_("Sorry, but the name must not contain < or >"));
 	else
 	{
 #ifndef GTK2
