@@ -1,8 +1,8 @@
 /*
- * $Id: infobox.c,v 1.19 2001/12/24 12:13:04 tal197 Exp $
+ * $Id: infobox.c,v 1.21 2002/01/21 17:03:51 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -231,7 +231,10 @@ static GtkWidget *make_clist(guchar *path, DirItem *item, xmlNode *about)
 	struct stat	info;
 	char		*data[] = {NULL, NULL, NULL};
 	xmlNode 	*prop;
-	GtkTargetEntry 	target_table[] = { {"STRING", 0, 0} };
+	GtkTargetEntry 	target_table[] = {
+		{"STRING", 0, 0},
+		{"COMPOUND_TEXT", 0, 0},	/* XXX: Treats as STRING */
+	};
 
 	table = GTK_CLIST(gtk_clist_new(2));
 	GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(table), GTK_CAN_FOCUS);
