@@ -1,8 +1,8 @@
 /*
- * $Id: menu.c,v 1.179 2001/12/29 14:58:52 tal197 Exp $
+ * $Id: menu.c,v 1.182 2002/01/23 13:33:06 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -1133,6 +1133,9 @@ static gboolean link_cb(guchar *initial, guchar *path)
 		report_error("symlink: %s", g_strerror(errno));
 		return FALSE;
 	}
+
+	dir_check_this(path);
+
 	return TRUE;
 }
 
@@ -1587,9 +1590,15 @@ static void close_window(gpointer data, guint action, GtkWidget *widget)
 
 static void mini_buffer(gpointer data, guint action, GtkWidget *widget)
 {
+	MiniType type = (MiniType) action;
+
 	g_return_if_fail(window_with_focus != NULL);
 
-	minibuffer_show(window_with_focus, (MiniType) action);
+	/* Item needs to remain selected... */
+	if (type == MINI_SHELL)
+		window_with_focus->temp_item_selected = FALSE;
+
+	minibuffer_show(window_with_focus, type);
 }
 
 void menu_rox_help(gpointer data, guint action, GtkWidget *widget)
