@@ -1,8 +1,8 @@
 /*
- * $Id: support.c,v 1.65 2001/12/26 20:33:21 tal197 Exp $
+ * $Id: support.c,v 1.71 2002/01/18 15:39:07 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -47,11 +47,15 @@
 #include "my_vfs.h"
 #include "fscache.h"
 
+#ifndef GTK2
+# include "gconvert.h"
+#endif
+
 static GHashTable *uid_hash = NULL;	/* UID -> User name */
 static GHashTable *gid_hash = NULL;	/* GID -> Group name */
 
 /* Static prototypes */
-#ifdef GTK2
+#if defined(GTK2) || defined(THUMBS_USE_LIBPNG)
 static void MD5Transform(guint32 buf[4], guint32 const in[16]);
 #endif
 
@@ -814,23 +818,27 @@ guchar *shell_escape(guchar *word)
  */
 gboolean is_sub_dir(char *sub, char *parent)
 {
-	struct stat	parent_info;
+	struct stat parent_info;
 
-	/* These were pathdup()s, but that follows symlinks!
-	 * I guess they were here for a reason - something is probably
-	 * broken now...
-	 */
-	sub = g_strdup(sub);
-	
-	if (mc_stat(parent, &parent_info))
+	if (mc_lstat(parent, &parent_info))
 		return FALSE;		/* Parent doesn't exist */
 
+	/* For checking Copy/Move operations do a realpath first on sub
+	 * (the destination), since copying into a symlink is the same as
+	 * copying into the thing it points to. Don't realpath 'parent' though;
+	 * copying a symlink just makes a new symlink.
+	 * 
+	 * When checking if an icon depends on a file (parent), use realpath on
+	 * sub (the icon) too.
+	 */
+	sub = pathdup(sub);
+	
 	while (1)
 	{
 		char	    *slash;
 		struct stat info;
 		
-		if (mc_stat(sub, &info) == 0)
+		if (mc_lstat(sub, &info) == 0)
 		{
 			if (info.st_dev == parent_info.st_dev &&
 				info.st_ino == parent_info.st_ino)
@@ -1024,7 +1032,7 @@ char *readlink_dup(char *source)
 	return g_strndup(path, got);
 }
 
-#ifdef GTK2
+#if defined(GTK2) || defined(THUMBS_USE_LIBPNG)
 /*
  * This code implements the MD5 message-digest algorithm.
  * The algorithm is due to Ron Rivest. The original code was
@@ -1270,7 +1278,64 @@ char *md5_hash(char *message)
 	MD5Update(&ctx, message, strlen(message));
 	return MD5Final(&ctx);
 }
-#endif /* GTK2 */
+#endif /* GTK2 or THUMBS_USE_LIBPNG */
+
+/* Removes trailing / chars and converts a leading '~/' (if any) to
+ * the user's home dir. g_free() the result.
+ */
+gchar *icon_convert_path(gchar *path)
+{
+	guchar		*retval;
+	int		path_len;
+
+	g_return_val_if_fail(path != NULL, NULL);
+
+	path_len = strlen(path);
+	while (path_len > 1 && path[path_len - 1] == '/')
+		path_len--;
+	
+	retval = g_strndup(path, path_len);
+
+	if (path[0] == '~' && (path[1] == '\0' || path[1] == '/'))
+	{
+		guchar *tmp = retval;
+
+		retval = g_strconcat(home_dir, retval + 1, NULL);
+		g_free(tmp);
+	}
+
+	return retval;
+}
+
+/* Convert string 'src' from the current locale to UTF-8.
+ * If conversion fails, try again using ISO8859-1 -> UTF-8.
+ */
+gchar *to_utf8(gchar *src)
+{
+	gchar *retval;
+
+	if (!src)
+		return NULL;
+
+	retval = g_locale_to_utf8(src, -1, NULL, NULL, NULL);
+	if (retval)
+		return retval;
+
+	return g_convert(src, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+}
+
+/* Convert string 'src' to the current locale from UTF-8 */
+gchar *from_utf8(gchar *src)
+{
+	gchar *retval;
+	
+	if (!src)
+		return NULL;
+
+	retval = g_locale_from_utf8(src, -1, NULL, NULL, NULL);
+
+	return retval ? retval : g_strdup(src);
+}
 
 /****************************************************************
  *                      INTERNAL FUNCTIONS                      *
