@@ -1,8 +1,8 @@
 /*
- * $Id: usericons.c,v 1.26 2001/12/20 17:45:11 tal197 Exp $
+ * $Id: usericons.c,v 1.29 2002/01/21 14:29:51 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2001, the ROX-Filer team.
+ * Copyright (C) 2002, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -51,12 +51,6 @@
 #include "action.h"
 #include "display.h"
 
-/* Store glob-to-icon mappings */
-typedef struct _GlobIcon {
-	guchar *pattern;
-	guchar *iconpath;
-} GlobIcon;
-
 static GHashTable *glob_icons = NULL; /* Pathname -> Icon pathname */
 
 /* Static prototypes */
@@ -117,6 +111,8 @@ void read_globicons()
 		/* Handle the new XML file format */
 		for (node = root->xmlChildrenNode; node; node = node->next)
 		{
+			gchar *path;
+
 			if (node->type != XML_ELEMENT_NODE)
 				continue;
 			if (strcmp(node->name, "rule") != 0)
@@ -128,7 +124,10 @@ void read_globicons()
 			if (!match)
 				continue;
 
-			g_hash_table_insert(glob_icons, match,
+			path = icon_convert_path(match);
+			g_free(match);
+
+			g_hash_table_insert(glob_icons, path,
 					xmlNodeGetContent(icon));
 		}
 
